# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+

from typing import Any

class MockSelection:
    model = ...  # type: Any
    treeiter = ...  # type: int
    def __init__(self, num_fields) -> None: ...
    def get_selected(self): ...
    def clear(self): ...

class MockResponse:
    content_json = ...  # type: Any
    status_code = ...  # type: int
    def __init__(self, content_json: Any = ...): ...
    @property
    def content(self) -> str: ...
    def json(self): ...
    def text(self): ...

class MockOAuth:
    response = ...  # type: Any
    def __init__(self, response: Any = ...) -> None: ...
    def get(self, url): ...
    def authorization_url(self, auth_endpoint, code_challenge_method, code_challenge): ...
    def post(self, url, data): ...

class MockBuilder:
    objects = ... # type: dict
    def get_object(self, o): ...

class VerifyMock:
    def verify(self, *args, **kwargs): ...


class MockDialog:
    return_code = ... # type: int
    def run(self): ...
    def hide(self): ...

