from typing import Tuple

eduvpn_main_logo = ...  # type: str
eduvpn_name = ...  # type: str
lets_connect_main_logo = ...  # type: str
lets_connect_name = ...  # type: str


def get_brand(lets_connect: bool) -> Tuple[str, str]: ...
