# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+
import nacl.signing
from typing import Optional

def common_name_from_cert(pem_data: str) -> str: ...
def gen_code_verifier(length: Optional[int] = 128): ...
def gen_code_challenge(code_verifier): ...
def make_verifier(key: str) -> nacl.signing.VerifyKey: ...
def gen_base32(length: int = ...) -> str: ...