# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+

from typing import Any, Optional, Tuple
from pil import Image
import gi
from gi.repository import GdkPixbuf

def make_unique_id(): ...
def error_helper(parent, msg_big, msg_small): ...
def thread_helper(func): ...
def bytes2pixbuf(data, width: Any = ..., height: Any = ..., display_name: Optional[Any] = ...): ...
def pil2pixbuf(img: Image.Image) -> GdkPixbuf.Pixbuf: ...
def get_prefix(): ...
def have_dbus(): ...
def get_pixbuf(logo: str): ...
def metadata_of_selected(builder): ...
def detect_distro() -> Tuple[str, str]: ...
def are_we_running_ubuntu1804() -> bool: ...