# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+
from eduvpn.metadata import Metadata

def insert_config(settings): ...
def list_providers(): ...
def store_provider(meta: Metadata, config_dict: dict) -> str: ...
def delete_provider(uuid): ...
def connect_provider(uuid): ...
def list_active(): ...
def disconnect_provider(uuid): ...
def is_provider_connected(uuid): ...
def update_config_provider(meta: Metadata, config_dictr: dict) -> None: ...
def update_keys_provider(uuid, cert, key): ...
def monitor_all_vpn(callback): ...
def monitor_vpn(uuid, callback): ...
def disconnect_all(): ...