# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+

def new_provider(builder, verifier, secure_internet_uri: str, institute_access_uri: str, lets_connect: bool): ...
