# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+

from typing import Optional
from eduvpn.metadata import Metadata

def browser_step(builder, meta: Metadata, verifier, lets_connect: bool,
                 force_token_refresh: Optional[bool] = False): ...
def _phase1_background(meta: Metadata, dialog, verifier, builder,force_token_refresh: bool,
                       lets_connect: bool) -> None: ...
def _phase1_callback(meta:  Metadata, port, code_verifier, oauth, auth_url, dialog, builder, state,
                     lets_connect: bool) -> None: ...
def _show_dialog(dialog, auth_url, builder) -> None: ...
def _phase2_background(meta:  Metadata, port, oauth, code_verifier, auth_url, dialog, builder, state,
                       lets_connect: bool) -> None: ...
def _phase2_callback(meta:  Metadata, oauth, dialog, builder, lets_connect: bool) -> None: ...
