# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+

from typing import Any

others_path = ... # type: str
config_path = ...  # type: str
providers_path = ...  # type: str
secure_internet_uri = ...  # type: str
institute_access_uri = ...  # type: str
secure_internet_uri_dev = ...  # type: str
institute_access_uri_dev = ...  # type: str
verify_key = ...  # type: str
verify_key_dev = ...  # type: str
locale = ...  # type: str
icon_size = ...  # type: Any
