# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+

from eduvpn.metadata import Metadata
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


def activate_connection(meta: Metadata, builder: Gtk.Builder, verifier, lets_connect: bool): ...
def _auth_check(oauth, meta: Metadata, verifier, builder: Gtk.Builder, lets_connect: bool) -> None: ...
def _fetch_updated_config(oauth, meta: Metadata, verifier, builder: Gtk.Builder ) -> None: ...
def _2fa_check(meta, builder, oauth, config_dict, info) -> None: ...
def _cert_check(meta: Metadata, oauth, builde: Gtk.Builderr, info) -> None: ...