# python-eduvpn-client - The GNU/Linux eduVPN client and Python API
#
# Copyright: 2017, The Commons Conservancy eduVPN Programme
# SPDX-License-Identifier: GPL-3.0+

def _background(meta, dialog, builder, config_dict: dict, lets_connect: bool): ...
def finalizing_step(builder, meta, config_dict: dict, lets_connect: bool): ...
